# This program may be used, executed, copied, modified and distributed
# without royalty for the purpose of developing, using, marketing, or distribution

#-----------------------------------------------------------------
# InstallAdministrativeObjects.py
#-----------------------------------------------------------------
#
#  This script is generated by the WPS/WID migration tools when migrating a WICS repository to WPS.
#  This script will define DataSources in WPS corresponding to DB connection pools defined in the WICS repository.
#  This script will define DataSources in WPS corresponding to Relationship DBs defined in the WICS repository.
#  This script will define Relationships/Roles in WPS corresponding to those defined in the WICS repository.
#  This script will define Scheduler Entries in WPS corresponding to scheduler entries defined in the WICS repository.
#    
#  This script can be included in the wsadmin command invocation like this:
#  
#     wsadmin -lang jython -f InstallAdministrativeObjects.py
#
#  The script expects 0 parameters:
#    
#
#-----------------------------------------------------------------
from com.ibm.wbiserver.scheduler.common import *
from java.text import SimpleDateFormat
import javax.management as mgmt 

#-----------------------------------------------------------------
# setup_Globals()
#
#-----------------------------------------------------------------
def setup_Globals():
   global AdminConfig

   global JNDI_PREFIX
   global DATASOURCE_DESCRIPTION
   global DB2_JDBC_PROVIDER_TYPE
   global DB2_JDBC_PROVIDER_NAME
   global MS_SQL_JDBC_PROVIDER_TYPE
   global MS_SQL_JDBC_PROVIDER_NAME
   global ORACLE_JDBC_PROVIDER_TYPE
   global ORACLE_JDBC_PROVIDER_NAME
   global SCRIPT_NAME
   global DATE_FORMATTER

   JNDI_PREFIX = "jdbc/wbi60migration/"
   DATASOURCE_DESCRIPTION = "WBI 6.0 Migration Datasource"

   DB2_JDBC_PROVIDER_TYPE = "DB2 Universal JDBC Driver Provider (XA)"
   DB2_JDBC_PROVIDER_NAME = "WBI 6.0 Migration DB2 Universal JDBC Driver Provider (XA)"

   MS_SQL_JDBC_PROVIDER_TYPE = "WebSphere embedded ConnectJDBC driver for MS SQL Server (XA)"
   MS_SQL_JDBC_PROVIDER_NAME = "WBI 6.0 Migration WebSphere embedded ConnectJDBC driver for MS SQL Server (XA)"

   ORACLE_JDBC_PROVIDER_TYPE = "Oracle JDBC Driver (XA)"
   ORACLE_JDBC_PROVIDER_NAME = "WBI 6.0 Migration Oracle JDBC Driver (XA)"

   SCRIPT_NAME = "InstallAdministrativeObjects.py: "
   DATE_FORMATTER = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSz")

#---------------------------------------------------------
# getConfigObject(configObjectType, configObjectAttrName, configObjectAttrValue)
#
# Get the config object of specified type and specified attribute value
# returns "" if not found
#---------------------------------------------------------   
def getConfigObject(configObjectType, configObjectAttrName, configObjectAttrValue):
   cfgObjs = AdminConfig.list(configObjectType).split(lineSeparator)

   for cfgObj in cfgObjs:
      if cfgObj != '' :
         cfgObjName = AdminConfig.showAttribute(cfgObj, configObjectAttrName)
         if cfgObjName == configObjectAttrValue :
            return cfgObj
   return ""

#-----------------------------------------------------------------
# getAuthenticationAlias(dsName, dbLogin, dbPassword)
#
#-----------------------------------------------------------------
def getAuthenticationAlias(dsName, dbLogin, dbPassword):
   aliasName = dbLogin+"_"+dsName
   #---------------------------------------------------------
   #Check if Alias already exists 
   #---------------------------------------------------------   
   authData = getConfigObject("JAASAuthData", "alias", aliasName)
   if authData != "" :
      return authData

   #---------------------------------------------------------
   # If not, then create it
   #---------------------------------------------------------
   #---------------------------------------------------------
   # Lookup the parent security object
   #---------------------------------------------------------
   securityParent = AdminConfig.list("Security").split(lineSeparator)[0]
   attrs = []
   attr = ["alias", aliasName]
   attrs.append(attr)
   attr = ["userId", dbLogin]
   attrs.append(attr)
   attr = ["password", dbPassword]
   attrs.append(attr)
   attr = ["description", "WBI 6.0 alias for datasource " + dsName]
   attrs.append(attr)

   authAlias = AdminConfig.create("JAASAuthData", securityParent, attrs)

   print SCRIPT_NAME + "JAASAuthData \"" + aliasName + "\" created successfully"
   return authAlias

#-----------------------------------------------------------------
# get_JDBC_provider(jdbcProviderName, jdbcProviderType)
#
#-----------------------------------------------------------------
def get_JDBC_provider(jdbcProviderName, jdbcProviderType):
   #---------------------------------------------------------
   #Check if provider already exists 
   #---------------------------------------------------------   
   provider = getConfigObject("JDBCProvider", "name", jdbcProviderName)
   if provider != "" :
      return provider

   #---------------------------------------------------------
   # If not, then create it at the Cell scope
   #---------------------------------------------------------
   #---------------------------------------------------------
   # Get the config id for a JDBC Template 
   #---------------------------------------------------------
   template = AdminConfig.listTemplates("JDBCProvider", jdbcProviderType).split(lineSeparator)[0]
   if len(template) == 0:
      print SCRIPT_NAME + "Could not find a JDBCProvider template of type \"" + jdbcProviderType + "\""
      return
 
   #---------------------------------------------------------
   # Get the config id for the cell 
   #---------------------------------------------------------
   cell = AdminConfig.list("Cell")
 
   #---------------------------------------------------------
   # Create a new JDBCProvider using the template 
   #---------------------------------------------------------
   nameAttr = ["name", jdbcProviderName]
   attrs = []
   attrs.append(nameAttr)
   jdbcProvider = AdminConfig.createUsingTemplate("JDBCProvider", cell, attrs, template)

   print SCRIPT_NAME + "JDBCProvider \"" + jdbcProviderName + "\" created successfully"
   return jdbcProvider

#-----------------------------------------------------------------
# create_datasource
#-----------------------------------------------------------------
def create_datasource(dsName, dsJndiName, dsDescription, jdbcProviderName, jdbcProviderType, dbLogin, dbPassword, dbMinConnections, dbMaxConnections, dbServer, dbPort, dbName):

   #---------------------------------------------------------
   #if datasource already exists, then print out warning and return
   #---------------------------------------------------------
   dataSource = getConfigObject("DataSource", "name", dsName)
   if dataSource != "" :
      print SCRIPT_NAME + "DataSource " + dsName + " already exists"
      #---------------------------------------------------------
      # TODO: should check that all attributes match what we would be setting up
      #---------------------------------------------------------
      return

   #---------------------------------------------------------
   # else create the DataSource
   #---------------------------------------------------------
   #---------------------------------------------------------
   # Get the JDBC Provider 
   #---------------------------------------------------------
   jdbcProvider = get_JDBC_provider(jdbcProviderName, jdbcProviderType)

   #-----------------------------------------------------------------
   # Get the Authentication Alias
   #-----------------------------------------------------------------
   authDataAlias = getAuthenticationAlias(dsName, dbLogin, dbPassword)

   #-----------------------------------------------------------------
   # Setup the DataSource Attributes
   #-----------------------------------------------------------------
   attrs = []
   attr = ["name", dsName]
   attrs.append(attr)
   attr = ["jndiName", dsJndiName]
   attrs.append(attr)
   attr = ["description", dsDescription]
   attrs.append(attr)
   attr = ["authDataAlias", AdminConfig.showAttribute(authDataAlias, "alias")]
   attrs.append(attr)

   #-----------------------------------------------------------------
   # Set J2EE Resource Properties for accessing the Database
   #-----------------------------------------------------------------
   resProps = []
   attr = [["name", "databaseName"], ["value", dbName], ["type", "java.lang.String"], ["description", "This is a required property"]]
   resProps.append(attr)
   attr = [["name", "serverName"], ["value", dbServer], ["type", "java.lang.String"]]
   resProps.append(attr)
   attr = [["name", "portNumber"], ["value", dbPort], ["type", "java.lang.String"]]
   resProps.append(attr)
   attr = [["name", "password"], ["value", dbPassword], ["type", "java.lang.String"]]
   resProps.append(attr)
   psAttr = ["propertySet", [["resourceProperties", resProps]]]
   attrs.append(psAttr)

   #-----------------------------------------------------------------
   # Create the DataSource
   #-----------------------------------------------------------------
   newDS = AdminConfig.create("DataSource", jdbcProvider, attrs)

   #-----------------------------------------------------------------
   # Modify ConnectionPool Properties min and max connections
   #-----------------------------------------------------------------
   conPool = AdminConfig.showAttribute(newDS, "connectionPool")

   attrs = []
   attr = ["minConnections", dbMinConnections]
   attrs.append(attr)
   attr = ["maxConnections", dbMaxConnections]
   attrs.append(attr)

   AdminConfig.modify(conPool,attrs)

   print SCRIPT_NAME + "DataSource \"" + dsName + "\" created successfully"

   #-----------------------------------------------------------------
   # Save all changes
   #-----------------------------------------------------------------
   AdminConfig.save()

#-----------------------------------------------------------------
# Create Relationship
#-----------------------------------------------------------------
def create_relationship(relName, jndiName, isCached):

   #-----------------------------------------------------------------
   # Don't create if it already exists
   #-----------------------------------------------------------------
   relObject = getConfigObject("Relationship", "relName", relName)
   if relObject != "" :
      print SCRIPT_NAME + "Relationship: ", relName, " already exists"
      return

   #-----------------------------------------------------------------
   # Get the Relationship Service
   #-----------------------------------------------------------------
   cfgObjs = AdminConfig.list("RelationshipService").split(lineSeparator)
   relService = cfgObjs[0]

   #-----------------------------------------------------------------
   # Setup Attribute Values
   #-----------------------------------------------------------------
   attrs = []
   attr = ["relName", relName]
   attrs.append(attr)
   attr = ["datasource", jndiName]
   attrs.append(attr)
   attr = ["infrastructureVersion", "4.2.0"]
   attrs.append(attr)
   attr = ["isCached", isCached]
   attrs.append(attr)

   #-----------------------------------------------------------------
   # Create the Relationship
   #-----------------------------------------------------------------
   relObject = AdminConfig.create("Relationship", relService, attrs);

   #-----------------------------------------------------------------
   # Save all changes
   #-----------------------------------------------------------------
   AdminConfig.save()

   print SCRIPT_NAME + "Create Relationship: ", relName, "created"

#-----------------------------------------------------------------
# Create Role
#-----------------------------------------------------------------
def create_role(relName, roleName, runTimeSProcName, runTimeSProcOwnerName, runTimeTableName, runTimeTableOwnerName):

   #-----------------------------------------------------------------
   # Get the Relationship
   #-----------------------------------------------------------------
   relObject = getConfigObject("Relationship", "relName", relName)
   if relObject == "" :
      print SCRIPT_NAME + "Relationship: ", relName, "does not exist"
      print SCRIPT_NAME + "Could not create Relationship: ", relName, " Role: ", roleName
      return

   #-----------------------------------------------------------------
   # Don't create Role if it already exists
   #-----------------------------------------------------------------
   relRoles = AdminConfig.list("Role", relObject).split(lineSeparator)
   roleObject = ""
   for rl in relRoles:
      if rl != '' :
         rlName = AdminConfig.showAttribute(rl, "roleName")
         if rlName == roleName :
            print SCRIPT_NAME + "Relationship: ", relName, " Role: ", roleName, " already exists"
            return
      
   #-----------------------------------------------------------------
   # Setup Attribute Values
   #-----------------------------------------------------------------
   attrs = []
   attr = ["roleName", roleName]
   attrs.append(attr)
   attr = ["runTimeSProcName", runTimeSProcName]
   attrs.append(attr)
   attr = ["runTimeTableName", runTimeTableName]
   attrs.append(attr)
   attr = ["runTimeSProcOwnerName", runTimeSProcOwnerName]
   attrs.append(attr)
   attr = ["runTimeTableOwnerName", runTimeTableOwnerName]
   attrs.append(attr)

   #-----------------------------------------------------------------
   # Create the Role
   #-----------------------------------------------------------------
   roleObject = AdminConfig.create("Role", relObject, attrs)

   #-----------------------------------------------------------------
   # Save all changes
   #-----------------------------------------------------------------
   AdminConfig.save()

   print SCRIPT_NAME + "Relationship: ", relName, " Role: ", roleName, "created"

#-----------------------------------------------------------------
# Create Attribute
#-----------------------------------------------------------------
def create_attribute(relName, roleName, attrName):

   #-----------------------------------------------------------------
   # Get the Relationship
   #-----------------------------------------------------------------
   relObject = getConfigObject("Relationship", "relName", relName)
   if relObject == "" :
      print SCRIPT_NAME + "Relationship: ", relName, "does not exist"
      print SCRIPT_NAME + "Could not create Relationship: ", relName, " Role: ", roleName
      return

   #-----------------------------------------------------------------
   # Get the Role 
   #-----------------------------------------------------------------
   relRoles = AdminConfig.list("Role", relObject).split(lineSeparator)
   roleObject = ""
   for rl in relRoles:
      if rl != '' :
         rlName = AdminConfig.showAttribute(rl, "roleName")
         if rlName == roleName :
            roleObject = rl

   if roleObject == "" :
      print SCRIPT_NAME + "Relationship: ", relName, " Role: ", roleName, "does not exist"
      print SCRIPT_NAME + "Relationship: ", relName, " Role: ", roleName, " Attribute: ", attrName, "not created"
      return

   #-----------------------------------------------------------------
   # Don't create Attribute if it already exists
   #-----------------------------------------------------------------
   roleAttrs = AdminConfig.list("AttributeColumn", roleObject).split(lineSeparator)
   for at in roleAttrs:
      if at != '' :
         atName = AdminConfig.showAttribute(at, "attributeName")
         if atName == attrName :
            print SCRIPT_NAME + "Relationship: ", relName, " Role: ", roleName, " Attribute: ", attrName, " already exists"
            return

   #-----------------------------------------------------------------
   # Setup Attribute Values
   #-----------------------------------------------------------------
   attrs = []
   attr = ["attributeName", attrName]
   attrs.append(attr)
   attr = ["columnName", attrName]
   attrs.append(attr)

   #-----------------------------------------------------------------
   # Create the Attribute
   #-----------------------------------------------------------------
   attrObject = AdminConfig.create("AttributeColumn", roleObject, attrs)

   #-----------------------------------------------------------------
   # Save all changes
   #-----------------------------------------------------------------
   AdminConfig.save()

   print SCRIPT_NAME + "Relationship: ", relName, " Role: ", roleName, " Attribute: ", attrName, "created"

#-----------------------------------------------------------------
# Create Scheduler Entry
#-----------------------------------------------------------------
def create_scheduler_entry(enabled, action, componentName, componentType, initialDateStr, recurrenceType, recurrencePeriod, recurrencenWeeks):

   action = action.upper()
   recurrenceType = recurrenceType.upper()

   #-----------------------------------------------------------------
   # Initialize return value
   #-----------------------------------------------------------------
   result = ScheduleEntry()
	
   #-----------------------------------------------------------------
   # Set component name
   #-----------------------------------------------------------------
   result.setCName(componentName)
		
   #-----------------------------------------------------------------
   # Set Action
   #-----------------------------------------------------------------
   if action == "START":
      result.setCTransition(ScheduleEntry.SCHEDULER_ACTION_START_APPLICATION)
   elif action == "STOP":
      result.setCTransition(ScheduleEntry.SCHEDULER_ACTION_STOP_APPLICATION);
   elif action == "PAUSE":
      result.setCTransition(ScheduleEntry.SCHEDULER_ACTION_PAUSE_APPLICATION);
   else:
      print SCRIPT_NAME + "Illegal Argument: action=",action
      print SCRIPT_NAME + "Scheduler Entry for ", componentName, " not created"
      return
		
   #-----------------------------------------------------------------
   # Set initial date
   #-----------------------------------------------------------------
   initialDate = DATE_FORMATTER.parse(initialDateStr)

   result.setInitialDate(initialDate);
		
   #-----------------------------------------------------------------
   # Set recurrence Type
   #-----------------------------------------------------------------
   if recurrenceType == "MINUTES":
      result.setRType(Recurrence.MINUTES)
   elif recurrenceType == "HOURS":
      result.setRType(Recurrence.HOURS)
   elif recurrenceType == "DAYS":
      result.setRType(Recurrence.DAYS)
   elif recurrenceType == "WEEKS":
      result.setRType(Recurrence.WEEKS)
   elif recurrenceType == "MONTHS":
      result.setRType(Recurrence.MONTHS)
   elif recurrenceType == "YEARS":
      result.setRType(Recurrence.YEARS)
   elif recurrenceType == "MONDAY":
      result.setRType(Recurrence.MONDAY)
   elif recurrenceType == "TUESDAY":
      result.setRType(Recurrence.TUESDAY)
   elif recurrenceType == "WEDNESDAY":
      result.setRType(Recurrence.WEDNESDAY)
   elif recurrenceType == "THURSDAY":
      result.setRType(Recurrence.THURSDAY)
   elif recurrenceType == "FRIDAY":
      result.setRType(Recurrence.FRIDAY)
   elif recurrenceType == "SATURDAY":
      result.setRType(Recurrence.SATURDAY)
   elif recurrenceType == "SUNDAY":
      result.setRType(Recurrence.SUNDAY)
   elif recurrenceType == "LAST_DAY_OF_MONTH":
      result.setRType(Recurrence.LAST_DAY_OF_THE_MONTH)
   elif recurrenceType == "UNDEFINED":
      # Don't set RType
      pass
   else:
      print SCRIPT_NAME + "Illegal Argument: recurrenceType=",recurrenceType
      print SCRIPT_NAME + "Scheduler Entry for ", componentName, " not created"
      return
		
   #-----------------------------------------------------------------
   # Set recurrence Period
   #-----------------------------------------------------------------
   result.setRPeriod(recurrencePeriod);
		
   #-----------------------------------------------------------------
   # Set recurrence weeks
   #-----------------------------------------------------------------
   result.setRWeekNumber(recurrencenWeeks);

   #-----------------------------------------------------------------
   # Set enabled/disabled
   #-----------------------------------------------------------------
   if enabled :
      result.setScheduleEntryStatus(ScheduleEntry.SCHEDULED)
   else:
      result.setScheduleEntryStatus(ScheduleEntry.SUSPENDED)
   
   #-----------------------------------------------------------------
   # Get the MBean and invoke the createScheduleEntry method
   #-----------------------------------------------------------------
   schedMBeanName = AdminControl.queryNames("WebSphere:name=WBISchedulerMB1,platform=dynamicproxy,WBIScheduler=WBIScheduler,type=WBIScheduler,mbeanIdentifier=WBISchedulerMB1,*")
   schedMBeanObject = mgmt.ObjectName(schedMBeanName) 
   methodName = "createScheduleEntry"
   args = [result]
   schedEntry = ScheduleEntry()
   signature = [schedEntry.class.getName()]
   try:
      AdminControl.invoke_jmx(schedMBeanObject, methodName, args, signature)
   except java.lang.Exception,e:
      e.printStackTrace()
      print SCRIPT_NAME + "Scheduler Entry for ", componentName, " not created"
      return
   print SCRIPT_NAME + "Scheduler Entry for ", componentName, " created"

#-----------------------------------------------------------------
# Main
#-----------------------------------------------------------------
setup_Globals()

#-----------------------------------------------------------------
# Create DataSources and Relationships for migrated WICS Relationships
# Syntax:
#    create_datasource(dsName, dsJndiName, dsDescription, dbLogin, dbPassword, dbMinConnections, dbMaxConnections, dbServer, dbPort, dbName, jdbcProviderName, jdbcProviderType)
#    create_relationship(relName, jndiName, isCached)
#    create_role(relName, roleName, runTimeSProcName, runTimeSProcOwnerName, runTimeTableName, runTimeTableOwnerName)
#    create_attribute(relName, roleName, attrName)
#-----------------------------------------------------------------
print SCRIPT_NAME
print SCRIPT_NAME + "========================================================"
print SCRIPT_NAME + "Create DataSources and Relationships for migrated WICS Relationships"
print SCRIPT_NAME + "========================================================"

#-----------------------------------------------------------------
# Create DataSources for WICS DB Connection Pools
# Syntax:
#    create_datasource(dsName, dsJndiName, dsDescription, jdbcProviderName, jdbcProviderType, dbLogin, dbPassword, dbMinConnections, dbMaxConnections, dbServer, dbPort, dbName):
#-----------------------------------------------------------------
print SCRIPT_NAME
print SCRIPT_NAME + "========================================================"
print SCRIPT_NAME + "Create DataSources for migrated WICS DB Connection Pools"
print SCRIPT_NAME + "========================================================"

#-----------------------------------------------------------------
# Create Scheduler Entries for WICS Scheduler Entries
# Syntax:
#    create_datasource(dsName, dsJndiName, dsDescription, dbLogin, dbPassword, dbMinConnections, dbMaxConnections, dbServer, dbPort, dbName, jdbcProviderName, jdbcProviderType):
#-----------------------------------------------------------------
print SCRIPT_NAME
print SCRIPT_NAME + "========================================================"
print SCRIPT_NAME + "Create Scheduler Entries for migrated WICS Scheduler Entries"
print SCRIPT_NAME + "========================================================"

