/**
 * BEGIN CMVC
 * *************************************************************************
 *
 *   Workfile: WBISRVR/ws/code/migration.wbi.ics.hapi/Access_EJB/DynamicRouting/gen/src/com/ibm/sel/RoutingSelector.java, wbi.migration.ics.hapi, WBI70.WBISRVR, of1016.04
 *   Last update: 07/12/20 16:55:32
 *   SCCS path, id: /family/botp/vc/10/7/2/8/s.05 1.9
 *
 * ************************************************************************
 * END CMVC
 * BEGIN COPYRIGHT
 * ************************************************************************
 * Licensed Materials - Property of IBM
 * 5724-L01, 5655-N53, 5724-I82, 5655-R15 
 * 
 * Copyright IBM Corporation 2004, 2007. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * DESCRIPTION:
 *
 * ***********************************************************************
 * END COPYRIGHT
 **/

package com.ibm.sel;

@SuppressWarnings("all")
public class RoutingSelector implements com.ibm.websphere.sca.ServiceImplAsync,
		com.ibm.websphere.sca.ServiceImplSync {
    public static final String COPYRIGHT="Copyright IBM Corporation 2004, 2007.";  //$NON-NLS-1$
    
    static final String SEL_QNAME  = "{http://www.ibm.com/xmlns/prod/websphere/wbi/SelectorComponentDef/6.0.0}SelectorComponentDef";
    static final String COMPONENT_TNS  = "http://DynamicRouting/com/ibm";
    static final String COMPONENT_NAME = "RoutingSelector";    
    static final String COMPONENT_TYPE = "SELT";    
	
	static com.ibm.wbiservers.common.event.EventLogger logger = new com.ibm.wbiservers.common.event.EventLogger(
			SEL_QNAME, COMPONENT_TNS, COMPONENT_NAME, 1);
	    
    public Object invoke(com.ibm.websphere.sca.scdl.OperationType operationType, Object input)
{
  logger.logEntry(operationType.getName(), logger.getParams(input, operationType, operationType.getInputType()));

  Object result = null;

  try
  {
    result = invokeTarget(operationType, input);
  }
  catch (RuntimeException rex)
  {
    logger.logFailure(operationType.getName(), rex);
    throw rex;
  }

  logger.logExit(operationType.getName(), logger.getParams(result, operationType, operationType.getOutputType()));

  return result;
}
    
    public void invokeAsync(com.ibm.websphere.sca.scdl.OperationType operationType, Object input, com.ibm.websphere.sca.ServiceCallback callback, com.ibm.websphere.sca.Ticket ticket)
{
  logger.logEntry(operationType.getName(), logger.getParams(input, operationType, operationType.getInputType()));

  Object result = null;
  Exception returnException = null;

  try
  {
    result = invokeTarget(operationType, input);
  }
  catch (RuntimeException rex)
  {
    logger.logFailure(operationType.getName(), rex);
    returnException = rex;
  }

  logger.logExit(operationType.getName(), logger.getParams(result, operationType, operationType.getOutputType()));

  if (callback != null)
  {
    callback.onInvokeResponse(ticket, result, returnException);
  }
}
    
    Object invokeTarget(com.ibm.websphere.sca.scdl.OperationType operationType, Object input)
{
  Object result = null;
  com.ibm.wbiserver.common.selection.InvocationTarget invocationTarg = null;

  java.util.List targets = selectTargets(operationType, input);

  if (targets == null || targets.size() == 0)
  {
    throw new com.ibm.wbiserver.common.exception.NoSelectionTargetException("No destination was found in the table of targets that meet the input selection criteria.  Add a default destination or provide a destination with a selection range that will match the input selection criteria.");
  }

  // The InvocationTarget class knows how to invoke different types of subclasses.
  // Different types of subclasses invoke different types of targets.  In the case
  // of BusinessRules, the subclass is BusinessRuleInvocationTarget, and it knows
  // how to invoke BusinessRules.  In the case of SCA components, the subclass is
  // SCAInvocationTarget, and it knows how to invoke SCA components.

  try
  {
    invocationTarg = (com.ibm.wbiserver.common.selection.InvocationTarget) targets.get(0);
  }
  catch (ClassCastException e)
  {
    throw new com.ibm.wbiserver.common.exception.SelectorException("A custom SelectionAlgorithm must return a list of subclasses of the InvocationTarget class.");
  }

  logger.logTargetId(operationType.getName(), invocationTarg.getTargetID());

  result = invocationTarg.invoke(COMPONENT_TNS, COMPONENT_NAME, operationType, input);

  return result;
}
	
	java.util.List selectTargets(
			com.ibm.websphere.sca.scdl.OperationType operationType, Object input) {
	   Object[] selectorParms = null;
	   com.ibm.wbiserver.common.selection.SelectionAlgorithm selector = null;
	   String operationName = operationType.getName();
	   boolean inputIsAWrapper = operationType.isWrapperType(operationType.getInputType());
	   
	   // Each operation can define its own selection algorithm and selection parameters.
	   // The next section of code is a big switch statement over the operation name.  The
	   // result of this section of code is that the variables selectorParms will contain
	   // the selection parameters and selector will contain the SelectionAlgorithm for 
	   // the particular operation.
	   
	   
	   try
	   {
		   // Setup a try/catch block to catch any ClassCastExceptions that 
		   // might be caused by a custom selector's failure to implement the
		   // SelectionAlgorithm interface.
	       selector = (com.ibm.wbiserver.common.selection.SelectionAlgorithm) com.ibm.wbiservers.common.selection.GenericSelector.getInstance();       
		} catch (ClassCastException e) {
			throw new com.ibm.wbiserver.common.exception.InvalidSelectionAlgorithmException(
					"Custom SelectionAlgorithms must implement the SelectionAlgorthm interface.");
		}
	        
	       if ("Route".equals(operationName))
	       {
		      selectorParms = getSelectionParmsForRoute(input, inputIsAWrapper);
           }
			else {
				throw new com.ibm.wbiserver.common.exception.SelectorException(
						"The operation " + operationName
								+ " is not supported by this component.");
			}
       
       logger.logSelectionKey(operationType.getName(), selectorParms);
       
        // Although the SelectionAlgorithm interface supports multicast, this component does not.
        // The third parameter tells the selection algorithm to return the first match only.
        // Therefore, the resulting List will contain either 0 or 1 entry.
        
	   return selector.selectTargets(COMPONENT_TNS, COMPONENT_NAME, COMPONENT_TYPE, operationName, selectorParms, false);
	}   	      
	    
    Object[] getSelectionParmsForRoute(Object input, boolean inputIsAWrapper)
    {
		  
	   // Add all input parameters to the selectorParms Object array. 
	   // Use the XPaths and/or code snippets to mine the input object
	   // for the selection parameters.
	   
      
       Object[] tempParms = 
		  {  
			 getXPathParmDefParameter("consumer", "", input, inputIsAWrapper) 
		  };
		  		     
       return tempParms;    	
    }    
	    
 	   
 		 Object getXPathParmDefParameter(String param, String xPath, Object input, boolean isInputAWrapper)
		   {

		      boolean xPathSpecified = (xPath != null && xPath.equals("") == false);
		      boolean paramSpecified = (param != null && param.equals("") == false);

		      // catches all the cases where there is neither an xPath nor a param

		      if (xPathSpecified == false && paramSpecified == false)
		      {
		         return input;
		      }

		      // catches all the additional cases where there is no xPath

		      if (xPathSpecified == false && paramSpecified == true)
		      {
		         if (isInputAWrapper)
		         {
		            return ((commonj.sdo.DataObject)input).get(param);
		         }
		         else
		         {
		            return input;
		         }
		      }

		      // for all of the rest of the cases, we know there is an xPath

		      // divide the remaining cases up by whether the input is a wrapper type or not

		      if (isInputAWrapper)
		      {
		         if (paramSpecified == false)
		         {
		            throw new com.ibm.wbiserver.common.exception.SelectorException("For wrapped input messages, the parameter attribute is required for the XPathParameterDef element.");
		         }
		         else
		         {
		            String xPathToUse = null;

		            if (xPath.startsWith("/") == true)
		            {
		               xPathToUse = param + xPath;
		            }
		            else
		            {
		               xPathToUse = param + "/" + xPath;
		            }

		            return ((commonj.sdo.DataObject)input).get(xPathToUse); 
		         }
		      }

		      // the input is not a wrapper type.  Since only xPath cases are left, throw an
		      // exception if the input is not a DataObject

		      if ((input instanceof commonj.sdo.DataObject) == false)
		      {
		         throw new com.ibm.wbiserver.common.exception.SelectorException("The input message must contain a valid DataObject.");
		      }

		      return ((commonj.sdo.DataObject)input).get(xPath);
		   }
}